package org.kamiblue.client.util.threads

import kotlinx.coroutines.CompletableDeferred
import kotlinx.coroutines.completeWith
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import kotlinx.coroutines.sync.Mutex
import kotlinx.coroutines.sync.withLock
import org.kamiblue.client.event.KamiEventBus
import org.kamiblue.client.event.events.RunGameLoopEvent
import org.kamiblue.client.util.Wrapper
import org.kamiblue.event.listener.listener

object MainThreadExecutor {
    private val jobs = ArrayList<MainThreadJob<*>>()
    private val mutex = Mutex()

    init {
        listener<RunGameLoopEvent.Start>(Int.MIN_VALUE) {
            runJobs()
        }

        KamiEventBus.subscribe(this)
    }

    private fun runJobs() {
        if (jobs.isEmpty()) return

        runBlocking {
            mutex.withLock {
                jobs.forEach {
                    launch { it.run() }
                }
                jobs.clear()
            }
        }
    }

    suspend fun <T> add(block: suspend () -> T) =
        MainThreadJob(block).apply {
            if (Wrapper.minecraft.isCallingFromMinecraftThread) {
                run()
            } else {
                mutex.withLock {
                    jobs.add(this)
                }
            }
        }.deferred

    private class MainThreadJob<T>(private val block: suspend () -> T) {
        val deferred = CompletableDeferred<T>()

        suspend fun run() {
            deferred.completeWith(
                runCatching { block.invoke() }
            )
        }
    }
}